// Copyright  2010 Software Verification Limited
// Copyright  2020 Software Verify Limited
//
// Author:	  Stephen Kellett
//
// Origin:    Original article this code was supplied with 
//			  https://www.softwareverify.com/software-verify-blog/?p=589
//
// License:   You can use this source code in any way you want, just as long as you do not put a GPL (or similar) license
//            on it that forces other people to give their source code away. If they want to keep it closed that should 
//            be their choice. An open source license where freedom is important allows people to do what they want, 
//            it doesn't force their hand to make them share. That is why GPL (or similar) is not acceptable.
//
// Warranty:  No warranty for fitness or suitability for use is granted, expressly or implicitly. Use at your own risk.
//
// Questions: stephen@softwareverify.com
// 
// addResourceToDLL.cpp : Utility for adding custom resources to EXE/DLL.
//

#include "stdafx.h"
#include <windows.h>
#include <tchar.h>
#include "stdio.h"
#include "stdlib.h"
#include "string.h"

int doWork(char	*moduleName,
		   char	*customResource,
		   char	*customType,
		   char	*customName,
		   int	quietMode)
{
	// open the custom resource file and load its contents into memory

	HANDLE	hFile;

	hFile = CreateFile(customResource, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hFile == INVALID_HANDLE_VALUE)
	{
		if (!quietMode)
		{
			printf("Unable to open custom resource\r\n");
		}
		return -2;
	}

	DWORD	fileSizeHi, fileSizeLow;
	BYTE	*bytes;

	fileSizeLow = GetFileSize(hFile, &fileSizeHi);
	bytes = new BYTE [fileSizeLow];
	if (bytes == NULL)
	{
		if (!quietMode)
		{
			printf("Unable to allocate space for custom resource\r\n");
		}
		return -3;
	}

	DWORD	numRead = 0;

	if (ReadFile(hFile, bytes, fileSizeLow, &numRead, NULL) == 0 ||
		numRead == 0)
	{
		if (!quietMode)
		{
			printf("Unable read data from custom resource\r\n");
		}
		return -4;
	}

	HANDLE	hUpdateRes;													// update resource handle 
	BOOL	result; 

	// Open the file to which you want to add the dialog box resource. 
	
	hUpdateRes = BeginUpdateResource(moduleName, 
									 FALSE);							// do not delete existing resources
	if (hUpdateRes == NULL) 
	{ 
		if (!quietMode)
		{
			printf("Unable open existing resources in %s\r\n", moduleName);
		}
		return -5;
	} 
	
	// Add the dialog box resource to the update list. 
	
	result = UpdateResource(hUpdateRes,									// update resource handle 
							customType,									// change dialog box resource 
							customName,									// dialog box name 
							MAKELANGID(LANG_NEUTRAL, SUBLANG_NEUTRAL),  // neutral language
							bytes,										// ptr to resource info 
							fileSizeLow);								// size of resource info. 
	if (result == FALSE) 
	{ 
		if (!quietMode)
		{
			printf("Could not add resources to %s\r\n", moduleName);
		}
		return -6;
	} 
	
	// Write changes to the input DLL and then close it
	
	if (!EndUpdateResource(hUpdateRes, FALSE)) 
	{ 
		if (!quietMode)
		{
			printf("Could not write resources to %s\r\n", moduleName);
		}
		return -7;
	} 
	
	// tidy up

	delete [] bytes;

	return 0;
}

#define _COPYRIGHT_DATES	_T("2020-2022")

void printHelp()
{
	printf(_T("addResourceToDLL V1.0\r\n"));
	printf(_T("Copyright  ") _COPYRIGHT_DATES _T(" Software Verify Limited\r\n"));
	printf(_T("https://www.softwareverify.com\r\n"));
	printf(_T("\r\n"));
	printf(_T("Mandatory arguments:\r\n"));
	printf(_T("-moduleName pathToDLL (or EXE)\r\n"));
	printf(_T("-customResource pathToCustomResource\r\n"));
	printf(_T("-customType type\r\n"));
	printf(_T("-customName name\r\n"));
	printf(_T("\r\n"));
	printf(_T("Optional arguments:\r\n"));
	printf(_T("-quiet\r\n"));
	printf(_T("\r\n"));
	printf(_T("Example: addResourceToDLL -moduleName c:\\lottery.dll -customResource c:\\testData\\lotteryNumbers.dat -customType lottery -customName lottery_numbers\r\n"));
}

int main(int	argc, 
		 char	*argv[])
{
	// input arguments:
	// -moduleName "full path to module.dll"
	// -customResource "full path to custom resource"
	// -customType type of custom resource
	// -customName name of custom resource

	char	*moduleName = NULL;
	char	*customResource = NULL;
	char	*customType = NULL;
	char	*customName = NULL;
	int		quietMode = FALSE;

	int	i;

	for(i = 0; i < argc; i++)
	{
		if (strcmp(argv[i], "-moduleName") == 0)
		{
			i++;
			moduleName = argv[i];
		}
		else if (strcmp(argv[i], "-customResource") == 0)
		{
			i++;
			customResource = argv[i];
		}
		else if (strcmp(argv[i], "-customType") == 0)
		{
			i++;
			customType = argv[i];
		}
		else if (strcmp(argv[i], "-customName") == 0)
		{
			i++;
			customName = argv[i];
		}
		else if (strcmp(argv[i], "-quiet") == 0)
		{
			i++;
			quietMode = TRUE;
		}
	}

	// check if we have enough arguments

	if (moduleName == NULL ||
		customResource == NULL ||
		customType == NULL ||
		customName == NULL)
	{
		if (!quietMode)
		{
			printf("Not enough arguments.\r\n\r\n");
			printHelp();
		}
		return -1;
	}

	int	r;
	
	r = doWork(moduleName, customResource, customType, customName, quietMode);
	if (r == 0)
	{
		if (!quietMode)
		{
			printf("Added resource to DLL\r\n");
		}
	}

	return r;
}
